
#include "LL_Types.h"

#define START_SCAN_RUNNING					1 

#pragma pack(push, 4)
typedef struct _SCAN_OBJECT{ 
	#define CMND_SINGLE_CHAN_SCAN			1
	#define CMND_MULTI_CHAN_SCAN			2  
	#define CMND_MULTI_CHAN_CAL_SCAN		3  
	#define CMND_SINGLE_CHAN_DIGIN_SCAN		4
	#define CMND_MULTI_CHAN_DIGIN_SCAN		5    
	#define CMND_MULTI_CHAN_DIGIN_CAL_SCAN  6    

	BYTE SO_bScanType;
	DOUBLE SO_dblDevRate;
	UINT SO_uiTotalPointsReadByDrvr;  
	UINT SO_uiPointsInBuffer;  

	#define SCAN_DATA_REQUEST_SUCCEEDED		0x2  
	#define SCAN_ENDING						0x4 
	#define SCAN_HALTED						0x8 
	#define SCAN_CHECKSUM_ERROR				0x10
	#define SCAN_VOLTAGE_BUFFER_WRAP		0x20 
	#define SCAN_MICRO_CODE_BUFF_WRAP		0x40
	#define SCAN_DEVICE_IO_ERROR			0x80 
	#define SCAN_DEVICE_UNRECOVERABLE_ERROR	0x100 
	#define SCAN_DEVICE_NOT_FOUND			0x200 
	UINT SO_uiStatus;  

	UINT SO_uiSizeVoltageArray;  
	UINT SO_uiCurHead;  
	UINT SO_uiCurTail;  
	BYTE SO_bDigitalInput;  
	BYTE SO_bChecksum; 
	BYTE SO_bDeviceDisconnect;  
	UINT SO_uiScanStartTime;  
	UINT SO_uiScanEndTime;  
	BYTE SO_bScanArg;
}
SCAN_OBJECT, *PSCAN_OBJECT; 


// SCAN DATA STRUCTS
typedef struct _SCAN_DATA_USB_SINGLE{ 
	UINT	bDataChan0;
	BYTE	bDiginChan0;
}
SCAN_DATA_USB_SINGLE, *PSCAN_DATA_USB_SINGLE; 


typedef struct _SCAN_DATA_USB{ 
	UINT	bDataChan0;
	BYTE	bDiginChan0;
	UINT	bDataChan1;
	BYTE	bDiginChan1;
	UINT	bDataChan2;
	BYTE	bDiginChan2;
	UINT	bDataChan3;
	BYTE	bDiginChan3;
	UINT	bDataChan4;
	BYTE	bDiginChan4;
	UINT	bDataChan5;
	BYTE	bDiginChan5;
	UINT	bDataChan6;
	BYTE	bDiginChan6;
	UINT	bDataChan7;
	BYTE	bDiginChan7;
}
SCAN_DATA_USB, *PSCAN_DATA_USB; 



// INFO STRUCTS
typedef struct _SCAN_INFO_USB_PNTS_SINGLE{
	USHORT usDevID;
	USHORT usNumPntsToRead;
	UINT uiNumPntsInBuff; 
	UINT uiStatusFlags; 
	BYTE bLastDigin;
}
SCAN_INFO_USB_PNTS_SINGLE, *PSCAN_INFO_USB_PNTS_SINGLE; 

typedef struct _SCAN_INFO_USB{
	USHORT usDevID;
	USHORT usNumScansMax;
	USHORT usNumScansMin;
	BOOLEAN fOkayToReadLess;
	UINT uiStatusFlags; 
	UINT uiNumScansInBuff; 
	USHORT usNumScansRead; 
	BYTE bLastDigin;
}
SCAN_INFO_USB, *PSCAN_INFO_USB; 
#pragma pack(pop)



typedef BOOLEAN (CALLBACK *pSP_DisableDialog)(void);  
typedef BOOLEAN (CALLBACK *pSP_GetRegDevList)(USHORT* usCompleteDevList, UINT uiUnused);  

typedef BOOLEAN(CALLBACK *pSP_InitAllDevices_Thread)( USHORT* pausDevList, DOUBLE* pdblDevRate, BOOLEAN fShowDevList);  
typedef BOOLEAN(CALLBACK *pSP_GetInitThreadStat)(void);  
typedef BOOLEAN(CALLBACK *pSP_GetLastInitStat)(USHORT* pDevList, UINT* puiFlags);  
typedef BOOLEAN(CALLBACK *pSP_GetLastInitDevAndRates)(USHORT* pDevList, double* pdblRates);  

typedef BOOLEAN(CALLBACK *pSP_InitAllDevices)( USHORT* pausDevList, BYTE* pbNumDevices, DOUBLE* padblRate, BOOLEAN fShowDevList );  

typedef BOOLEAN(CALLBACK *pSP_GetOneConversion)( USHORT usDevID, UINT* uiConvRawCount, BYTE bCurChan, BYTE* pbDigIn); 
typedef BOOLEAN(CALLBACK *pSP_SendChan)(USHORT usDevID, BYTE bCurChan, BYTE* pbLastDigin); 
typedef BOOLEAN(CALLBACK *pSP_SendDigOut)(USHORT usDevID, BYTE bDigOut, BYTE* pbLastDigin); 
typedef BOOLEAN(CALLBACK *pDLL_StartScan)(USHORT usDevID, PSCAN_OBJECT pScanObj, BYTE bStartMode); 
typedef BOOLEAN(CALLBACK *pSP_EndScan)(USHORT usDevID); 
typedef BOOLEAN(CALLBACK *pDLL_ReadScanData)(USHORT usDevID, UINT* DynaVoltageBuff, USHORT numPointsToRead, PSCAN_OBJECT pScanObjO); 
typedef BOOLEAN(CALLBACK *pDLL_ReadScanDataWithDigin)(USHORT usDevID, UINT* DynaVoltageBuff, BYTE* bDiginBuff, BYTE* pbCurChan, USHORT numPointsToRead, PSCAN_OBJECT pScanObj); 

typedef BOOLEAN(CALLBACK *pSP_WriteWhileScanning)(USHORT usDevID, BYTE bCmnd, BYTE bArg, UINT* puiCurWriteStat); 

typedef BOOLEAN(CALLBACK *pSP_SendRate)(DOUBLE* dblRate, USHORT usDevID); 
typedef BOOLEAN(CALLBACK *pSP_SendDAC)(USHORT usDevID, DOUBLE dblVoltage, BYTE bDAC, BYTE* pbReserved); 

typedef BOOLEAN(CALLBACK *pSP_SystemCalibration)(USHORT usDevID, BYTE bCurChan); 

typedef BOOLEAN(CALLBACK *pSP_GetDevStat)(USHORT usDevID, UINT* _uiDevStat); 
typedef BOOLEAN(CALLBACK *pSP_CheckPower)(USHORT usDevID, BYTE bCurChan, BYTE* pbLastDigin, UINT* puiFlags); 



struct DEVICES{
	USHORT usDevID;
	BOOLEAN fInitialized;
	BOOLEAN fScanning;
	BOOLEAN fIsDiginScan;
	BOOLEAN fIsMultiChanScan;
	BYTE bCurChan;
	BYTE bLastDigin;
	DOUBLE dblRate;
//	UINT uiOS_Code;
	UINT uiLastErrorCodeThisDev;
	SCAN_OBJECT ScanObject;
};



class MyLL_USB
{
	private: 

		UINT uiOS_Code;

		BOOLEAN fIsValidStructPntr[32];

 		DEVICES* structDevices[32];

		BOOLEAN GetDevListIndex(USHORT usDevID, BYTE* bIndex);
		UINT	uiLastAnyError;
		BOOLEAN fBaseInitDone;

		HINSTANCE	hDLL;
		BOOLEAN		fAnyDevInit;
		USHORT		usDevList_GLOBAL[32];
		BOOLEAN		fInitFlags[32];

		pSP_DisableDialog			SP_DisableDialog;
		pSP_GetRegDevList			SP_GetRegDevList;
		pSP_InitAllDevices			SP_InitAllDevices;

		pSP_InitAllDevices_Thread	SP_InitAllDevices_Thread;
		pSP_GetInitThreadStat		SP_GetInitThreadStat;
		pSP_GetLastInitStat			SP_GetLastInitStat;
		pSP_GetLastInitDevAndRates	SP_GetLastInitDevAndRates;

		pSP_GetOneConversion		SP_GetOneConversion;
		pSP_SendChan				SP_SendChan;
		pSP_SendDigOut				SP_SendDigOut;
		pDLL_StartScan				DLL_StartScan;
		pDLL_ReadScanData			DLL_ReadScanData;
		pDLL_ReadScanDataWithDigin	DLL_ReadScanDataWithDigin;
		pSP_EndScan					SP_EndScan;

		pSP_SendRate				SP_SendRate;
		pSP_SendDAC					SP_SendDAC;

		pSP_SystemCalibration		SP_SystemCalibration;

		pSP_GetDevStat				SP_GetDevStat;
		pSP_CheckPower				SP_CheckPower;

		pSP_WriteWhileScanning		SP_WriteWhileScanning;

		BOOLEAN BaseVarInit();
		BOOLEAN GetInitStat();
		
		BOOLEAN GetOsVer_PRIVATE();
		BOOLEAN LoadDLL_PRIVATE();
		BOOLEAN GetDevList_PRIVATE(USHORT* pausDevList, BYTE* _bNumDevices);
		BOOLEAN GetZeroIndexDev_PRIVATE(USHORT* _usDevID);

		//* Starts scan for all currently supported type of public start scan function calls
		BOOLEAN StartScan_ALL_TYPES(BYTE bNumDevices, USHORT* pausDevID, BYTE* pabScanType, BYTE* pabNumChans, BYTE* pabScanStarted);

		//* Reads scan scan data for all currently supported type of public read scan function 
		//  calls that use NON-STRUCTURED type args when making the  call 
		BOOLEAN GetScanData(USHORT usDevID, USHORT usNumPntsToReadMax,  USHORT usNumPntsToReadMin, USHORT* usNumPntsRead, UINT* uiDataBuff, BOOLEAN fOkayToReadLess, UINT* uiStatusFlags, UINT* uiPntsInBuff, BYTE* bDiginBuff, BYTE* bLastDigin);

		//* Reads scan scan data for all currently supported type of public read scan function 
		//  calls that use STRUCTURED type args when making the  call 
		BOOLEAN GetScanData_USE_STRUCT( PSCAN_INFO_USB pScanInfo, PSCAN_DATA_USB paScanPacket);

		BOOLEAN MyLL_USB::GetScanData_USE_STRUCT( PSCAN_INFO_USB_PNTS_SINGLE pScanInfoPntsSingle, 
			DOUBLE* padblVoltages);

		//  init single device only
		BOOLEAN Init_PRIVATE(USHORT usDevID, BYTE bChan, DOUBLE* dblRate);
		//* init multiple devices (non-threaded)
        BOOLEAN Init_PRIVATE(USHORT* pausDevList, BYTE* pabChans, DOUBLE* padblRates);
		//* init multiple devices (threaded)
		BOOLEAN Init_PRIVATE(USHORT* pausDevList, BYTE* pabChans, DOUBLE* padblRates, BOOLEAN fThreaded);
   public:

		//* constructor
		MyLL_USB(); 
		//* destructor
		~MyLL_USB();

		//  init single device only - first device found, channel 0, 100Hz.
		BOOLEAN Init_SP();

		BOOLEAN GetDefDev_SP(USHORT* usDevID);

		//  init single device only
		BOOLEAN Init(USHORT usDevID);
		//  init single device only
		BOOLEAN Init(USHORT usDevID, BYTE bChan, DOUBLE* dblRate);
		//* init multiple devices (non-threaded)
        BOOLEAN Init(USHORT* pausDevList, BYTE* pabChans, DOUBLE* padblRates);
		//* init multiple devices (threaded)
		//7-10-02 BOOLEAN Init(USHORT* pausDevList, BYTE* pabChans, DOUBLE* padblRates, UINT _uiOS_Code, BOOLEAN fThreaded);
		BOOLEAN Init(USHORT* pausDevList, BYTE* pabChans, DOUBLE* padblRates, BOOLEAN fThreaded);

		BOOLEAN SetCurChan_SP(BYTE bChan);
		BOOLEAN SetCurChan(USHORT usDevID, BYTE bChan);
		BOOLEAN GetCurChan(BYTE* pbCurChan);
		BOOLEAN GetCurChan(USHORT usDevID, BYTE* pbCurChan);

        BOOLEAN SendDigitalOutput_SP(BYTE bDigOutVal);
        BOOLEAN SendDigitalOutput(USHORT usDevID, BYTE bDigOutVal);
        BOOLEAN GetDigitalInput_SP(BYTE* pbDiginVal);
        BOOLEAN GetDigitalInput(USHORT usDevID, BYTE* pbDiginVal);

        BOOLEAN GetOneCount(USHORT usDevID, UINT* puiRawCount);
        BOOLEAN GetOneVoltage_SP(DOUBLE* pdblVoltage);
        BOOLEAN GetOneVoltage(USHORT usDevID, DOUBLE* pdblVoltage);
        BOOLEAN GetOneVoltage(USHORT usDevID, DOUBLE* pdblVoltage, DOUBLE dblRange);

		BOOLEAN SendAnalogOut(BYTE bDAC, DOUBLE dblVoltage);
		BOOLEAN SendAnalogOut(USHORT usDevID, BYTE bDAC, DOUBLE dblVoltage);

		BOOLEAN ChangeCurDevRate_SP(DOUBLE* pdblRate);
		BOOLEAN ChangeCurDevRate(USHORT usDevID, DOUBLE* pdblRate);

		BOOLEAN Calibrate_SP();
		BOOLEAN Calibrate(USHORT usDevID);

		BOOLEAN GetLastDevError_SP(UINT* puiErrorCode);
		BOOLEAN GetLastDevError(UINT usDevID, UINT* puiErrorCode);
		BOOLEAN GetLastAnyError_SP(UINT* puiErrorCode);
		BOOLEAN GetLastAnyError(UINT* puiErrorCode);

		BOOLEAN GetDevStatus_SP(UINT* puiConnectionStat, UINT* puiPowerStat);
		BOOLEAN GetDevStatus(USHORT usDevID, UINT* puiConnectionStat, UINT* puiPowerStat);

		BOOLEAN GetCurDevInitStatus(USHORT usDevID);
		BOOLEAN GetCurDevInitStatus_SP();

		BOOLEAN GetCurDevScanStatus(USHORT usDevID, BOOLEAN* fOkayToStartScan, BOOLEAN* fScanning);
		BOOLEAN GetCurDevScanStatus_SP(BOOLEAN* pfOkayToStartScan, BOOLEAN* pfScanning);

		//10-14-02
		//* writes to the device while it is scanning - in a very efficient way - good for digital output
		BOOLEAN WriteWhileScanning_SP(BYTE bCmnd, BYTE bArg, UINT* puiCurWriteStat);
		BOOLEAN WriteWhileScanning(USHORT usDevID, BYTE bCmnd, BYTE bArg, UINT* puiCurWriteStat);


		//* start scan single device single chan only
		BOOLEAN StartScan_SP();
		//* start scan single device single chan only
		BOOLEAN StartScan(USHORT usDevID);
		//* start scan single device any scan type
		BOOLEAN StartScan(USHORT usDevID, BYTE bScanType, BYTE bNumChans);


		//* start scan with options (recommend calling with only one device at a time to prevent buffer overflow.
		BOOLEAN StartScan(BYTE bNumDevices, USHORT* pausDevIDList, BYTE* pabScanTypeList, BYTE* pabNumChansList, BYTE* pabScanStartedList);


		//NOTE: In the "reads" below, when there is an argument (bLastDigin) included in the
		//  call, the value that will be in that variable upon return from the call. However
		//  when using a "digital input" type scan, that variable will still contain the "last"
		//  digital input obtained at the time of the request, while the digital inputs within
		//  the digital input array correspond directly to the digital input that was present
		//  at the time the corresponding channel in the data array was retrieved. So when using
		//  a digital input to trigger on a particular channel be sure to use the one within the
		//  digital input array rather than the other.

		//* read fixed amount -- no digin, structured args, +-5 voltage array, uses device found in Init_SP
		BOOLEAN GetScanVoltage_SP(PSCAN_INFO_USB_PNTS_SINGLE pScanInfoPntsSingle, DOUBLE* padblVoltages);

		//* Same as above except uses device passed as member in structure pointed to by pScanInfoPntsSingle
		BOOLEAN GetScanVoltage(PSCAN_INFO_USB_PNTS_SINGLE pScanInfoPntsSingle, DOUBLE* padblVoltages);

		//* read variable amount using single-chan type scan, without digin, and using structured 
		//  data. First arg is single structure containing per-request
		//  scan info from caller to callee and back. Second arg is array of structured data
		//  that represents one packet of all channels to be scanned.
		BOOLEAN GetScanData(PSCAN_INFO_USB pScanInfo, PSCAN_DATA_USB paScan);

		//* read fixed amount -- no digin, non-structured args.
		BOOLEAN GetScanData(USHORT usDevID, USHORT usNumPntsToRead, USHORT* pusNumPntsRead, UINT* uiDataBuff, UINT* uiStatusFlags, UINT* uiPntsInBuff, BYTE* bLastDigin);
		
		//* read variable amount -- no digin, non-structured args.
		BOOLEAN GetScanData(USHORT usDevID, USHORT usNumPntsToReadMax, USHORT usNumPntsToReadMin, USHORT* pusNumPntsRead, UINT* puiDataBuff, BOOLEAN fOkayToReadLess, UINT* puiStatusFlags, UINT* puiPntsInBuff, BYTE* pbLastDigin);
		
		//* read variable amount -- with digin, non-structured args.
		BOOLEAN GetScanData(USHORT usDevID, USHORT usNumPntsToReadMax, USHORT usNumPntsToReadMin, USHORT* pusNumPntsRead, UINT* pauiDataBuff, BYTE* pabDiginBuff, BOOLEAN fOkayToReadLess, UINT* puiStatusFlags, UINT* puiPntsInBuff, BYTE* pbLastDigin);
		

		//* end scan single device
		BOOLEAN EndScan_SP();
		//* end scan single device
		BOOLEAN EndScan(USHORT usDevID);
		//* end scan multiple devices
		BOOLEAN EndScan(BYTE bNumDevices, USHORT* pausDevList);

};


class MyLL_TCPIP
{ };

class MyLL_Serial
{ };

class LLABS : public MyLL_USB, MyLL_TCPIP, MyLL_Serial
{ };
